<?php
/**
 * Basic argument handler that parses a lon,lat,lon,lat argument, restricts
 * the main query by those coordinates and adjusts the style plugin bounding
 * box.
 *
 * @ingroup views_argument_handlers
 */
class nicemap_handler_argument_bbox extends views_handler_argument {
  function option_definition() {
    $options = parent::option_definition();
    $options['break_phrase'] = array('default' => TRUE);
    return $options;
  }

  function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);
    $form['title']['#access'] = FALSE;

    $form['behavior'] = array(
      '#title' => t('Behavior'),
      '#type' => 'select',
      '#options' => array(
        'limit' => t('Limit results'),
        'non-limit' => t("Don't limit results")
      ),
      '#description' => t('The bounding box can either limit the results that
                           are returned, simply describe the box to be used
                           for display.'),
      '#default_value' => $this->options['behavior'],
    );

    unset($form['wildcard']);
    unset($form['wildcard_substitution']);
  }

  /**
   * Override default_actions() to remove actions that don't
   * make sense for a null argument.
   */
  function default_actions($which = NULL) {
    if ($which) {
      if (in_array($which, array('ignore', 'not found', 'empty', 'default'))) {
        return parent::default_actions($which);
      }
      return;
    }
    $actions = parent::default_actions();
    unset($actions['summary asc']);
    unset($actions['summary desc']);
    return $actions;
  }

  function query() {
    // We would use this, except views_break_phrase() doesn't allow negative values.
    // views_break_phrase($this->argument, $this);

    $coords = explode(',', $this->argument);
    $minx = $miny = $maxx = $maxy = 0;
    if (count($coords) == 4) {
      $this->view->init_style();

      $bounds = array(
        'minx' => ($coords[0] < $coords[2]) ? $coords[0] : $coords[2],
        'miny' => ($coords[1] < $coords[3]) ? $coords[1] : $coords[3],
        'maxx' => ($coords[0] > $coords[2]) ? $coords[0] : $coords[2],
        'maxy' => ($coords[1] > $coords[3]) ? $coords[1] : $coords[3],
      );

      $projection = nicemap_get_projection(variable_get('nicemap_wms_crs', ''));
      $bounds = $projection->getmap($bounds, $this->view->style_plugin->options);

      $lat_field = !empty($this->view->style_plugin->options['fields']['latitude']) ? $this->view->style_plugin->options['fields']['latitude'] : '';
      $lon_field = !empty($this->view->style_plugin->options['fields']['longitude']) ? $this->view->style_plugin->options['fields']['longitude'] : '';

      $this->view->style_plugin->options['minx'] = $bounds['minx'];
      $this->view->style_plugin->options['miny'] = $bounds['miny'];
      $this->view->style_plugin->options['maxx'] = $bounds['maxx'];
      $this->view->style_plugin->options['maxy'] = $bounds['maxy'];

      if ($this->options['behavior'] != 'non-limit') {
        $lat_table = $this->view->field[$lat_field]->table;
        $lat_field = $this->view->field[$lat_field]->real_field;
        $this->query->add_where(0, "{$lat_table}.{$lat_field} >= %f AND {$lat_table}.{$lat_field} < %f", array($bounds['miny'], $bounds['maxy']));

        $lon_table = $this->view->field[$lon_field]->table;
        $lon_field = $this->view->field[$lon_field]->real_field;
        $this->query->add_where(0, "{$lon_table}.{$lon_field} >= %f AND {$lon_table}.{$lon_field} < %f", array($bounds['minx'], $bounds['maxx']));
      }
    }
  }
}
